#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>       // for waitpid
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#include <string.h>

#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <kiconloader.h>
#include "kbag.h"
#include "kpreference.h"
#include "kitem.h"

KBag::KBag(QWidget*parent,const char* name):KTMainWindow(name){

  MyConfig = kapp->getConfig();

  MyConfig->setGroup("General");
  compression=new QString(MyConfig->readEntry("compress", "gzip"));
  floppy_mount=new QString(MyConfig->readEntry("floppy_mount", "/mnt/fd0"));
  removable_mount=new QString(MyConfig->readEntry("removable_mount", "/mnt/removable"));
  hard_mount=new QString(MyConfig->readEntry("hard_mount", "/tmp"));

  list_names  = new QStrList();
  list_folders= new QStrList();
  list_targets= new QStrList();
  list_names   -> setAutoDelete(true);
  list_folders -> setAutoDelete(true);
  list_targets -> setAutoDelete(true);
  MyConfig->readListEntry("list_names", *list_names);
  MyConfig->readListEntry("list_folders", *list_folders);
  MyConfig->readListEntry("list_targets", *list_targets);


  list = new KMainList(this);
  list->insertStrList(list_names);

  QPopupMenu *file = new QPopupMenu;
  CHECK_PTR( file );
  file->insertItem(i18n("Exit"), this, SLOT(exit()));

  QPopupMenu *items = new QPopupMenu;
  CHECK_PTR( items);
  items->insertItem( i18n("Move to media"), this, SLOT(item_to()));
  items->insertItem( i18n("Move from media"), this, SLOT(item_from()));
  items->insertSeparator();
  items->insertItem( i18n("Bag's options"), this, SLOT(opt_item()));
  items->insertSeparator();
  items->insertItem( i18n("Add bag"), this, SLOT(add_item()));
  items->insertItem( i18n("Delete bag"), this, SLOT(del_item()));

  QPopupMenu *options = new QPopupMenu;
  CHECK_PTR( options );
  options->insertItem(i18n("Setup"), this, SLOT(preference()));

  QPopupMenu *help = new QPopupMenu;
  CHECK_PTR( help );
  help->insertItem( i18n("User manual"), this, SLOT(help_manual()));
  help->insertSeparator();
  help->insertItem( i18n("About"), this, SLOT(about()));

  // create menubar
  menuBar()->insertItem( i18n("File"), file );
  menuBar()->insertItem( i18n("Bags"), items );
  menuBar()->insertItem( i18n("Options"), options );
  menuBar()->insertSeparator();
  menuBar()->insertItem( i18n("Help"), help );

  // create toolbar
  QString* tempstr= new QString(KApplication::kde_datadir());
  tempstr->append("/kbag/kbag_add.xpm");
  QPixmap *xpm = new QPixmap(tempstr->data());
	toolBar()->insertButton( *xpm, 10000, SIGNAL(clicked()), this,
		SLOT(add_item()), true, i18n("Add bag"));
	tempstr->sprintf("%s/kbag/kbag_options.xpm", KApplication::kde_datadir().data());
  xpm->load(tempstr->data());
	toolBar()->insertButton( *xpm, 10001, SIGNAL(clicked()), this,
		SLOT(opt_item()), true, i18n("options for selected bag"));
	tempstr->sprintf("%s/kbag/kbag_del.xpm", KApplication::kde_datadir().data());
  xpm->load(tempstr->data());
	toolBar()->insertButton( *xpm, 10002, SIGNAL(clicked()), this,
		SLOT(del_item()), true, i18n("Delete bag"));
	toolBar()->insertSeparator();
	tempstr->sprintf("%s/kbag/kbag_to.xpm", KApplication::kde_datadir().data());
  xpm->load(tempstr->data());
	toolBar()->insertButton( *xpm, 10003, SIGNAL(clicked()), this,
		SLOT(item_to()), true, i18n("Move selected bag to removable media"));
	tempstr->sprintf("%s/kbag/kbag_from.xpm", KApplication::kde_datadir().data());
  xpm->load(tempstr->data());
	toolBar()->insertButton( *xpm, 10004, SIGNAL(clicked()), this,
		SLOT(item_from()), true, i18n("Move selected bag from removable media"));
	toolBar()->insertSeparator();
	tempstr->sprintf("%s/kbag/kbag_exit.xpm", KApplication::kde_datadir().data());
  xpm->load(tempstr->data());
	toolBar()->insertButton( *xpm, 10005, SIGNAL(clicked()), this,
		SLOT(exit()), true, i18n("Exit from programm"));

	// create popup menu for listbox
  popup_menu = new QPopupMenu;
  popup_menu->insertItem( i18n("Move to media"), this, SLOT(item_to()));
  popup_menu->insertItem( i18n("Move from media"), this, SLOT(item_from()));
  popup_menu->insertSeparator();
  popup_menu->insertItem( i18n("Bag's options"), this, SLOT(opt_item()));
  popup_menu->insertSeparator();
  popup_menu->insertItem( i18n("Add bag"), this, SLOT(add_item()));
  popup_menu->insertItem( i18n("Delete bag"), this, SLOT(del_item()));
}

KBag::~KBag(){
  MyConfig->setGroup("General");
  MyConfig->writeEntry("compress", compression->data());
  MyConfig->writeEntry("floppy_mount", floppy_mount->data());
  MyConfig->writeEntry("removable_mount", removable_mount->data());
  MyConfig->writeEntry("hard_mount", hard_mount->data());
  if (list_names->count()>0)
    {
    MyConfig->writeEntry("list_names",  *list_names);
    MyConfig->writeEntry("list_folders",*list_folders);
    MyConfig->writeEntry("list_targets",*list_targets);
    }
}

void KBag::resizeEvent ( QResizeEvent * e){
  list->setGeometry(5, 55, this->width()-10, this->height()-60);
}

void KBag::exit(void){
  close();
}

void KBag::about(void){
QMessageBox::about( this, i18n("About kBag"),
i18n("kBag v1.2.0\n\n                    Copyright 1999 Kir Smirnov\n                     Check for new versions at\nhttp://members.tripod.com/Kir_Smirnov/kBag/index.html\n\n                                     Kir Smirnov <ksm@mail.ru>"));
}

void KBag::preference(void){
	KPreference *new_dialog = new KPreference;
	new_dialog->show();
}

void KBag::add_item(void){
  how_add=1;            // add new item !!!
	KItem *new_dialog = new KItem;
	new_dialog->show();
}

void KBag::opt_item(void){
  how_add=2;            // options for selected item !!!
	if (list->currentItem()==-1)
		{
		QMessageBox::about(this, i18n("Bag's options"), i18n("Choose item and try again"));
		return;
		}
	KItem *new_dialog = new KItem;
	new_dialog->show();
	list->clear();
  list->insertStrList(list_names);
}

void KBag::del_item(void){
	if (list->currentItem()==-1)
		{
		QMessageBox::about(this, i18n("Delete bag"), i18n("Choose item and try again"));
		return;
		}
	// remove database file.
 	QString *configpath = new QString(KApplication::localkdedir());
  configpath->append("/share/apps/kBag/");
 	configpath->append(list_names->at(kbag->list->currentItem()));
 	::remove(configpath->data());
	// remove tables strings for deleted entry.
	list_names  ->remove(kbag->list->currentItem());
	list_folders->remove(kbag->list->currentItem());
	list_targets->remove(kbag->list->currentItem());
	list->clear();
  list->insertStrList(list_names);
}
/****************************************************************
*
*   Check for chenged files, tar, [g|b]zip and move to media...
*
*****************************************************************/
void KBag::item_to(void){
  char tempstr[255];              // temp string

  datalist.setAutoDelete( TRUE ); // delete items when they are removed

	if (list->currentItem()==-1)
		{
		QMessageBox::about(this, i18n("Move to media"), i18n("Choose item and try again"));
		return;
		}
  // here - mount device if needed (then not hard drive).
  if (strcmp(list_targets->at(list->currentItem()), "2")!=0)
    {
  	QMessageBox::information(this, i18n("Move to media"),
	    i18n("Insert new (clear) media and hit OK"), QMessageBox::Ok);
	  if (strcmp(list_targets->at(list->currentItem()), "1")!=0)
	    { // mount floppy
	    exec_argv[1]=floppy_mount->data();
	    }
	  else
	    { //mount removable
	    exec_argv[1]=removable_mount->data();
	    }
    exec_argv[0]="mount";
    exec_argv[2]=0;
 	  pid=fork();
    if (pid==0)
      {
 	    execvp("mount", exec_argv);
   	  }
    waitpid(pid, &status, 0);
	  }
  // check recursively
 	QString *configpath = new QString(KApplication::localkdedir());
  configpath->append("/share/apps/kBag/");
 	configpath->append(list_names->at(list->currentItem()));
item_to_100:
	if ((datafile=::open(configpath->data(), O_RDONLY))==-1)
  	{
	  if (QMessageBox::Retry==QMessageBox::warning(this, i18n("Error"),
	    i18n("Error opening database file\nEmail this situation to author!"),
		  QMessageBox::Abort, QMessageBox::Retry)) goto item_to_100;
 		goto item_to_umount;
  	}
  // read datafile into datalist...
  read(datafile, &templong, 4);
  if (templong!=1)
    {
  	QMessageBox::information(this, i18n("Move to media"),
  	  i18n("Read documentation for version update."), QMessageBox::Ok);
  	goto item_to_999;
    }
  while(4==read(datafile, &templong, 4))
    {
    read(datafile, &tempstr, templong);
    read(datafile, &templong, 4);
    datalist.append(new MyDatabase(tempstr, templong));
    }
  // check it !!! :)
  exec_argc=4;
 	item_to_add_folder(list_folders->at(list->currentItem()));
	// create output file
  exec_argv[0]="tar";
  exec_argv[1]="-cP";
  if (strcmp(compression->data(), "gzip")==0)
    exec_argv[2]="--gzip";
  else
    exec_argv[2]="--use-compress-program=bzip2";
	strcpy(tempstr, "-f");
	switch(strcmp(list_targets->at(list->currentItem()), "1"))
	{
	  case -1: strcat(tempstr, floppy_mount->data()); break;
	  case  0: strcat(tempstr, removable_mount->data()); break;
	  case  1: strcat(tempstr, hard_mount->data()); break;
	  default: break;
	}
	strcat(tempstr, "/");
	strcat(tempstr, list_names->at(list->currentItem()));
  if (strcmp(compression->data(), "gzip")==0)
  	strcat(tempstr, ".kbag.tgz");
  else
    strcat(tempstr, ".kbag.tar.bz2");
	exec_argv[3]=(char*)malloc(strlen(tempstr));
	strcpy(exec_argv[3],tempstr);
	exec_argv[exec_argc++]=0;
	if (exec_argc>5)
	  {
	  pid=fork();
	  if (pid==0)
	    {
  	  execvp("tar", exec_argv);
  	  }
    if (pid!=-1) waitpid(pid, &status, 0);
  	}
  // recalc database after moving...
  recalc();
  //
item_to_999:
 	::close(datafile);
  // here - unmount device...
item_to_umount:
  if (strcmp(list_targets->at(list->currentItem()), "2")!=0)
    {
	  if (strcmp(list_targets->at(list->currentItem()), "1")!=0)
	    { // mount floppy
	    exec_argv[1]=floppy_mount->data();
	    }
	  else
	    { // mount removable
	    exec_argv[1]=removable_mount->data();
	    }
    exec_argv[0]="umount";
    exec_argv[2]=0;
 	  pid=fork();
    if (pid==0)
      {
 	    execvp("umount", exec_argv);
   	  }
    waitpid(pid, &status, 0);
	  }
	QMessageBox::information(this, i18n("Move to media"),
	  i18n("Operation complete successful."), QMessageBox::Ok);
}
// helpful procedure for recursive check
void KBag::item_to_add_folder(const char* path){
DIR *cur_dir;             // for opendir
struct dirent *direntry;  // for readdir
QString *tempstr;         // temp. string

	tempstr = new QString;
		
item_to_add_100:	
	if ((cur_dir = opendir(path)) == NULL)
    {
		if (QMessageBox::Retry==QMessageBox::warning(this,
		  i18n("Error opening source directory"), path,
			QMessageBox::Abort, QMessageBox::Retry)) goto item_to_add_100;
		return;
	  }
//	QMessageBox::about(this, i18n("Debug message"), path);
	
	while ((direntry = readdir(cur_dir)) != NULL)
  	{
		tempstr->sprintf("%s/%s", path, direntry->d_name);
	  stat(tempstr->data(), &stat_buf);
	  if (0==strcmp(direntry->d_name, ".")) continue;  // we don't like this :)
	  if (0==strcmp(direntry->d_name, "..")) continue;
		if (S_ISDIR(stat_buf.st_mode))
			{
			item_to_add_folder(tempstr->data());
			goto item_to_add_200;
			}
		else
		  {
		  MyDatabase* cur_data;
		  for(cur_data=datalist.first(); cur_data != 0; cur_data=datalist.next())
		    {
		    if (0==strcmp(cur_data->path(), tempstr->data()))
		      {
  		    if (cur_data->time() < stat_buf.st_mtime)
  		      {
  		      exec_argv[exec_argc]=(char*)malloc(strlen(cur_data->path())+1);
		        strcpy(exec_argv[exec_argc],cur_data->path());
		        exec_argc++;
		        }
	        goto item_to_add_200;
		      }
		    }
      exec_argv[exec_argc]=(char*)malloc(strlen(tempstr->data())+1);
	    strcpy(exec_argv[exec_argc],tempstr->data());
	    exec_argc++;
		  }
item_to_add_200:;		
	  }
	closedir(cur_dir);
}
/*****************************************************************
*
*   Move files from media
*
*****************************************************************/
void KBag::item_from(void){
  char tempstr[255];              // temp string
	
	if (list->currentItem()==-1)
		{
		QMessageBox::about(this, i18n("Move from media"), i18n("Choose item and try again"));
		return;
		}
  // here - mount device if needed (then not hard drive).
  if (strcmp(list_targets->at(list->currentItem()), "2")!=0)
    {
  	QMessageBox::information(this, i18n("Move from media"),
	    i18n("Insert media with data and hit OK"), QMessageBox::Ok);
	  if (strcmp(list_targets->at(list->currentItem()), "1")!=0)
	    { // mount floppy
	    exec_argv[1]=floppy_mount->data();
	    }
	  else
	    { // mount removable
	    exec_argv[1]=removable_mount->data();
	    }
    exec_argv[0]="mount";
    exec_argv[2]=0;
 	  pid=fork();
    if (pid==0)
      {
 	    execvp("mount", exec_argv);
   	  }
    waitpid(pid, &status, 0);
	  }
	// update data !
	strcpy(tempstr, "-f");
	switch(strcmp(list_targets->at(list->currentItem()), "2"))
	{
	  case  1: strcat(tempstr, floppy_mount->data()); break;
	  case  0: strcat(tempstr, hard_mount->data()); break;
	  case -1: strcat(tempstr, removable_mount->data()); break;
	  default: break;
	}
	strcat(tempstr, "/");
	strcat(tempstr, list_names->at(list->currentItem()));
  if (strcmp(compression->data(), "gzip")==0)
  	strcat(tempstr, ".kbag.tgz");
  else
    strcat(tempstr, ".kbag.tar.bz2");
  exec_argv[0]="tar";
  exec_argv[1]="-xP";
  if (strcmp(compression->data(), "gzip")==0)
    exec_argv[2]="--gzip";
  else
    exec_argv[2]="--use-compress-program=bzip2";
	exec_argv[3]=(char*)malloc(strlen(tempstr));
	strcpy(exec_argv[3],tempstr);
	exec_argv[4]=0;
  pid=fork();
  if (pid==0)
    {
 	  execvp("tar", exec_argv);
 	  }
  waitpid(pid, &status, 0);
  // delete file after processing
	switch(strcmp(list_targets->at(list->currentItem()), "2"))
	{
	  case  1: strcpy(tempstr, floppy_mount->data()); break;
	  case  0: strcpy(tempstr, hard_mount->data()); break;
	  case -1: strcpy(tempstr, removable_mount->data()); break;
	  default: break;
	}
	strcat(tempstr, "/");
	strcat(tempstr, list_names->at(list->currentItem()));
  if (strcmp(compression->data(), "gzip")==0)
  	strcat(tempstr, ".kbag.tgz");
  else
    strcat(tempstr, ".kbag.tar.bz2");
  remove(tempstr);
  // recalc database after moving...
  recalc();
  // here - unmount device...
  if (strcmp(list_targets->at(list->currentItem()), "2")!=0)
    {
	  if (strcmp(list_targets->at(list->currentItem()), "1")!=0)
	    { // umount floppy
	    exec_argv[1]=floppy_mount->data();
	    }
	  else
	    { // umount removable
	    exec_argv[1]=removable_mount->data();
	    }
    exec_argv[0]="umount";
    exec_argv[2]=0;
 	  pid=fork();
    if (pid==0)
      {
 	    execvp("umount", exec_argv);
   	  }
    waitpid(pid, &status, 0);
	  }
	QMessageBox::information(this, i18n("Move from media"),
	  i18n("Operation complete successful."), QMessageBox::Ok);
}
/****************************************************************
*
*   Recalculate database for selected item (internal function)
*
*****************************************************************/
void KBag::recalc(void){

	if (list->currentItem()==-1)
		{
		QMessageBox::about(this, i18n("Recalc database"), 
      i18n("Can't found bag to recalc\nEmail this situation to author!"));
		return;
		}
	
	stat(kbag->list_folders ->at(kbag->list->currentItem()), &stat_buf);
	if (!S_ISDIR(stat_buf.st_mode))
	  {
	  QMessageBox::information(this, i18n("Error"),
	    i18n("I can't work with file as root entry\n"
      "Email this situation to author!"), QMessageBox::Ok);
	  }
	else
	  {
  	QString *configpath = new QString(KApplication::localkdedir());
	  configpath->append("/share/apps/kBag");
  	mkdir(configpath->data(), 0777);
	  configpath->append("/");
  	configpath->append(list_names->at(kbag->list->currentItem()));
recalc_100:
  	if ((datafile=::creat(configpath->data(), 0644))==-1)
	  	{
		  if (QMessageBox::Retry==QMessageBox::warning(this, i18n("Error"),
		    i18n("Error creating database file\nEmail this situation to author!"),
			  QMessageBox::Abort, QMessageBox::Retry)) goto recalc_100;
  		return;
	  	}
	  // write configfile version (current -1)
	  templong=1;
	  write(datafile, &templong, 4);
	  // write database recursively
  	recalc_add_folder(kbag->list_folders ->at(kbag->list->currentItem()));
  	::close(datafile);
  	}
}

void KBag::recalc_add_folder(const char *path){
DIR *cur_dir;             // for opendir
struct dirent *direntry;  // for readdir
QString *tempstr;         // temp. string

  tempstr = new QString;
		
recalc_200:	
	if ((cur_dir = opendir(path)) == NULL)
    {
		if (QMessageBox::Retry==QMessageBox::warning(this,
		  i18n("Error opening source directory"), path,
			QMessageBox::Abort, QMessageBox::Retry)) goto recalc_200;
		return;
	  }
//	QMessageBox::about(this, i18n("Debug message"), path);
	
	while ((direntry = readdir(cur_dir)) != NULL)
  	{
		tempstr->sprintf("%s/%s", path, direntry->d_name);
	  stat(tempstr->data(), &stat_buf);
	  if (0==strcmp(direntry->d_name, ".")) continue;
	  if (0==strcmp(direntry->d_name, "..")) continue;
		if (S_ISDIR(stat_buf.st_mode))
			{
			tempstr->sprintf("%s/%s", path,direntry->d_name);
			recalc_add_folder(tempstr->data());
			}
		else
		  {
    	templong=1+strlen(tempstr->data());
    	write(datafile, &templong, sizeof(templong));
    	write(datafile, tempstr->data(), templong);
    	write(datafile, &stat_buf.st_mtime, sizeof(stat_buf.st_mtime));
		  }
	  }
	closedir(cur_dir);
}

void KBag::help_manual(void){
  kapp->invokeHTMLHelp(QString("kbag/index.html"),QString(""));
}




